@@ -226,9 +226,6 @@ struct aws_h2_stream *aws_h2_stream_new_request(
     AWS_PRECONDITION(options);
 
     struct aws_h2_stream *stream = aws_mem_calloc(client_connection->alloc, 1, sizeof(struct aws_h2_stream));
-    if (!stream) {
-        return NULL;
-    }
 
     /* Initialize base stream */
     stream->base.vtable = &s_h2_stream_vtable;
@@ -368,7 +365,7 @@ static void s_stream_update_window(struct aws_http_stream *stream_base, size_t i
     if (!increment_size) {
         return;
     }
-    if (!connection->base.manual_window_management) {
+    if (!connection->base.stream_manual_window_management) {
         /* auto-mode, manual update window is not supported */
         AWS_H2_STREAM_LOG(
             DEBUG, stream, "Manual window management is off, update window operations are not supported.");
@@ -754,7 +751,7 @@ struct aws_h2err aws_h2_stream_on_decoder_headers_i(
         /* Client */
         if (name_enum == AWS_HTTP_HEADER_STATUS) {
             uint64_t status_code;
-            int err = aws_strutil_read_unsigned_num(header->value, &status_code);
+            int err = aws_byte_cursor_utf8_parse_u64(header->value, &status_code);
             AWS_ASSERT(!err && "Invalid :status value. Decoder should have already validated this");
             (void)err;
 
@@ -840,9 +837,27 @@ struct aws_h2err aws_h2_stream_on_decoder_push_promise(struct aws_h2_stream *str
     return AWS_H2ERR_SUCCESS;
 }
 
+static int s_stream_send_update_window(struct aws_h2_stream *stream, uint32_t window_size) {
+    struct aws_h2_frame *stream_window_update_frame =
+        aws_h2_frame_new_window_update(stream->base.alloc, stream->base.id, window_size);
+    if (!stream_window_update_frame) {
+        AWS_H2_STREAM_LOGF(
+            ERROR,
+            stream,
+            "WINDOW_UPDATE frame on stream failed to be sent, error %s",
+            aws_error_name(aws_last_error()));
+        return AWS_OP_ERR;
+    }
+
+    aws_h2_connection_enqueue_outgoing_frame(s_get_h2_connection(stream), stream_window_update_frame);
+    stream->thread_data.window_size_self += window_size;
+    return AWS_OP_SUCCESS;
+}
+
 struct aws_h2err aws_h2_stream_on_decoder_data_begin(
     struct aws_h2_stream *stream,
     uint32_t payload_len,
+    uint32_t total_padding_bytes,
     bool end_stream) {
 
     AWS_PRECONDITION_ON_CHANNEL_THREAD(stream);
@@ -873,22 +888,34 @@ struct aws_h2err aws_h2_stream_on_decoder_data_begin(
     }
     stream->thread_data.window_size_self -= payload_len;
 
-    /* send a stream window_update frame to automatically maintain the stream self window size, if
-     * manual_window_management is not set */
-    if (payload_len != 0 && !end_stream && !stream->base.owning_connection->manual_window_management) {
-        struct aws_h2_frame *stream_window_update_frame =
-            aws_h2_frame_new_window_update(stream->base.alloc, stream->base.id, payload_len);
-        if (!stream_window_update_frame) {
-            AWS_H2_STREAM_LOGF(
-                ERROR,
-                stream,
-                "WINDOW_UPDATE frame on stream failed to be sent, error %s",
-                aws_error_name(aws_last_error()));
+    if (total_padding_bytes != 0 && !end_stream && stream->base.owning_connection->stream_manual_window_management) {
+        /**
+         * Automatically update the flow-window to account for padding, even if "manual window management"
+         * is enabled. We do this because the current API doesn't have any way to inform the user about padding,
+         * so we can't expect them to manage it themselves.
+         */
+        if (s_stream_send_update_window(stream, total_padding_bytes)) {
             return aws_h2err_from_last_error();
         }
-
-        aws_h2_connection_enqueue_outgoing_frame(s_get_h2_connection(stream), stream_window_update_frame);
-        stream->thread_data.window_size_self += payload_len;
+        AWS_H2_STREAM_LOGF(
+            DEBUG,
+            stream,
+            "DATA with %" PRIu32
+            " padding. Updating the window for padding and one byte for padding length automatically for stream.",
+            total_padding_bytes - 1 /* one byte for padding length */);
+    }
+    /* send a stream window_update frame to automatically maintain the stream self window size, if
+     * manual_window_management is not set */
+    if (payload_len != 0 && !end_stream && !stream->base.owning_connection->stream_manual_window_management) {
+        if (s_stream_send_update_window(stream, payload_len)) {
+            return aws_h2err_from_last_error();
+        }
+        AWS_H2_STREAM_LOGF(
+            TRACE,
+            stream,
+            "Connection with no manual window management, updating window with size %" PRIu32
+            " automatically for stream.",
+            payload_len);
     }
 
     return AWS_H2ERR_SUCCESS;
